

	#ifndef	REGS_68HC908JK1

REGS_68HC908JK1	.EQU	1

PTA 	.EQU	$00
PTB 	.EQU 	$01
PTC 	.EQU 	$02
PTD 	.EQU	$03
	; D4:D5  TIM channels
	; D6:D7  25mA open-drain with pullup

DDRA 	.EQU	$04
DDRB 	.EQU	$05
DDRC 	.EQU	$06
DDRD 	.EQU	$07

PDCR	.EQU	$0A
PTAPUE	.EQU	$0D

KBSCR	.EQU	$1A
KBIER	.EQU	$1B

INTSCR	.EQU	$1D
CONFIG2	.EQU	$1E
CONFIG 	.EQU	$1F

TSC		.EQU	$20
TCNT	.EQU	$21
TCNTH		.EQU	$21
TCNTL		.EQU	$22
TMOD	.EQU	$23
TMODH		.EQU	$23
TMODL		.EQU	$24
TSC0	.EQU	$25
TCH0	.EQU	$26
TCH0H		.EQU	$26
TCH0L		.EQU	$27

TSC1	.EQU	$28
TCH1	.EQU	$29
TCH1H		.EQU	$29
TCH1L		.EQU	$2A

ADSCR	.EQU	$3C
ADR		.EQU	$3D
ADICLK	.EQU	$3E

BSR		.EQU	$FE00
RSR		.EQU	$FE01

BFCR		.EQU	$FE03
INT1		.EQU	$FE04
INT2		.EQU	$FE05
INT3		.EQU	$FE06

FLCR		.EQU	$FE08
FLBPR		.EQU	$FE09

BRKH		.EQU	$FE0C
BRKL		.EQU	$FE0D
BRKSCR		.EQU	$FE0E

COPCTL		.EQU	$FFFF


_vect_ADC		.EQU	$FFDE
_vect_KBIR		.EQU	$FFE0
_vect_TIMOVF	.EQU	$FFF2
_vect_TIM1		.EQU	$FFF4
_vect_TIM0		.EQU	$FFF6
_vect_IRQ		.EQU	$FFFA
_vect_SWI		.EQU	$FFFC
_vect_RESET		.EQU	$FFFE

	#endif

	#include	"bits68hc908.asm"


