
    .PROCESSOR 6502
    
    #include "macro_ipart0.asm"		/* head  declaration */

/* continuing the macro definition in another file does not work */

    #include "macro_ipart1.asm"		/* macro is ended here */


    .ORG 0

    headPart





