; 11include.inc - part of the symbolic include testing. 
; To test non-symbol compliant include filenames, the name needs to begin with digits.
; Please don't rename it.
;
; --Mike Saarna

 .byte $11
